/**
  * @file SignalRelayARM.c
  * @brief This module contains the implementation of the signal relay hardware
  * functions.
  *
  * @author Stefan Popovic
  *
  * @date 14/10/2016
  *
  * @version 0.1a
  *
  * @copyright GNU Public License
  *
  * Test configuration:
  *
  * MCU: STM32F107VC
  * Dev.Board: EasyMx Pro v7 for STM32 ARM
  * External Clock 72.0000 MHz
  * Ext. Modules: Signal Relay Click Board
  * SW: MikroC PRO for ARM v4.9.0
  *
  * NOTES:
    - Signal Relay click board is essentially board with 4 5VDC Signal Relays
    - Place Signal Relay click board on mikroBUS slot 1
    - Pull down on PORTC pins 8 and 9, on PORTD pins 8 and 9 (board specific)
    - Button Press level enabled SW10 PORTC/H and PORTD/H
    - RE 1: PA4
    - RE 2: PC2
    - RE 3: PD13
    - RE 4: PA0


  /**************************************************************************
  * Module Preprocessor Constants
  **************************************************************************/


  /**************************************************************************
  * Module Preprocessor Macros
  **************************************************************************/


  /**************************************************************************
  * Module Typedefs
  **************************************************************************/


   /**************************************************************************
   * Includes
   **************************************************************************/
   #include "built_in.h"
   #include "recources.h"
   #include <stdbool.h>

  /**************************************************************************
  * Module Variable Definitions
  **************************************************************************/

  char txt[24];
  bool oldC9;
  bool oldC8;
  bool oldD9;
  bool oldD8;

  // TFT module connections
  unsigned int TFT_DataPort at GPIOE_ODR;
  sbit TFT_RST at GPIOE_ODR.B8;
  sbit TFT_RS at GPIOE_ODR.B12;
  sbit TFT_CS at GPIOE_ODR.B15;
  sbit TFT_RD at GPIOE_ODR.B10;
  sbit TFT_WR at GPIOE_ODR.B11;
  sbit TFT_BLED at GPIOE_ODR.B9;
  // End TFT module connections

   void Display_Init()
  {
    TFT_Init_ILI9341_8bit(320, 240); // TFT Init
    TFT_BLED = 1;
   }

   // Draw Frame on TFT module
   void DrawFrame()
   {
    TFT_Fill_Screen(CL_WHITE);
    TFT_Set_Pen(CL_BLACK, 1);
    TFT_Set_Brush(1,CL_BLACK, 0, 0, CL_BLACK, CL_BLACK);
    TFT_Line(20, 220, 300, 220);
    TFT_Line(5,  46, 315,  46);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("Signal Relay  Click  Board  Example", 5, 14);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("EasyMx PRO v7 for STM32 ARM", 19, 223);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("www.mikroe.com", 200, 223);

    TFT_Write_Text("RELAY 1:", 10, 90 );
    TFT_Write_Text("RELAY 2:", 170, 90 );
    TFT_Write_Text("RELAY 3:", 10, 150);
    TFT_Write_Text("RELAY 4:", 170, 150 );

    TFT_Set_Pen(CL_RED, 5);
    TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);

    // Switch for RELAY 1
    TFT_Line(80, 100, 100, 100);
    TFT_Line(130, 100, 150, 100);
    TFT_Line(100, 100, 130, 80);     // OPEN switch

    // Switch for RELAY 2
    TFT_Line(240, 100, 260, 100);
    TFT_Line(290, 100, 310, 100);
    TFT_Line(260, 100, 290, 80);     // OPEN switch

    // Switch for RELAY 3
    TFT_Line(80, 155, 100, 155);
    TFT_Line(130, 155, 150, 155);
    TFT_Line(100, 155, 130, 135);    // OPEN switch

    // Switch for RELAY 4
    TFT_Line(240, 155, 260, 155);
    TFT_Line(290, 155, 310, 155);
    TFT_Line(260, 155, 290, 135);    // OPEN switch

  }

    void main()
    {
      Display_Init();
      DrawFrame();

      oldC9 = false;
      oldC8 = false;
      oldD9 = false;
      oldD8 = false;

      // Configure PORTC pins as input
      GPIO_Digital_Input (&GPIOC_BASE, 
                          _GPIO_PINMASK_8 | 
                          _GPIO_PINMASK_9);  
      // Configure PORTD pins as input
      GPIO_Digital_Input (&GPIOD_BASE, 
                          _GPIO_PINMASK_8 | 
                          _GPIO_PINMASK_9); 
      // Configure PORTA pins as output
      GPIO_Digital_Output(&GPIOA_BASE, 
                          _GPIO_PINMASK_4 | 
                          _GPIO_PINMASK_0 ); 
      // Configure PORTC pin as output                    
      GPIO_Digital_Output(&GPIOC_BASE, _GPIO_PINMASK_2);   

      // Configure PORTD pin as output               
      GPIO_Digital_Output(&GPIOD_BASE, _GPIO_PINMASK_13);                   
      
      // Setting pin values to zero
      GPIOA_ODR.B4 = 0;
      GPIOC_ODR.B2 = 0;
      GPIOD_ODR.D13 = 0;
      GPIOA_ODR.B0 = 0;

      while(1) {                             // endless loop

          // Relay 1
          if (Button(&GPIOC_IDR, 9, 1, 1))   // detect logical one on PORTC9 pin
            {

              if (!oldC9)                    // detect zero-to-one transition
              {
                Delay_ms(200);

                // marking
                TFT_Set_Pen(CL_WHITE, 5);
                TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);

                // OPEN switch
                TFT_Line(100, 100, 130, 80);       

                // showing
                TFT_Set_Pen(CL_RED, 5);
                TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);

                // CLOSED switch
                TFT_Line(100, 100, 130, 100);                              

                oldC9 = 1;
                GPIOA_ODR.B4 = 1;
               }
               else                          // detect one-to-zero transition
               {
                Delay_ms(200);

                //marking
                TFT_Set_Pen(CL_WHITE, 5);
                TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);

                // CLOSED switch
                TFT_Line(100, 100, 130, 100);                             

                // showing
                TFT_Set_Pen(CL_RED, 5);
                TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);

                 // OPEN switch
                TFT_Line(100, 100, 130, 80);                              

                oldC9 = 0;
                GPIOA_ODR.B4 = 0;
              }
          }
          
          // Relay 2
          if (Button(&GPIOD_IDR, 9, 1, 1))   // detect logical one on PORTD9 pin
            {

              if (!oldD9)                    // detect zero-to-one transition
              {
                Delay_ms(200);

                // marking
                TFT_Set_Pen(CL_WHITE, 5);
                TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);

                // OPEN switch
                TFT_Line(260, 100, 290, 80);   

                // showing
                TFT_Set_Pen(CL_RED, 5);
                TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);

                // CLOSED switch
                TFT_Line(260, 100, 290, 100); 

                oldD9 = 1;
                GPIOC_ODR.B2 = 1;

               }
               else                          // detect one-to-zero transition
               { 
                Delay_ms(200);

                // marking
                TFT_Set_Pen(CL_WHITE, 5);
                TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);

                // CLOSED switch
                TFT_Line(260, 100, 290, 100); 

                // showing
                TFT_Set_Pen(CL_RED, 5);
                TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);

                // OPEN switch
                TFT_Line(260, 100, 290, 80);   

                oldD9 = 0;
                GPIOC_ODR.B2 = 0;
               }
            }
          
          // Relay 3
          if (Button(&GPIOC_IDR, 8, 1, 1))   // detect logical one on PORTC8 pin
            {

              if (!oldC8)                    // detect zero-to-one transition
              {
                Delay_ms(200);

                // marking
                TFT_Set_Pen(CL_WHITE, 5);
                TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);

                // OPEN switch
                TFT_Line(100, 155, 130, 135); 

                // showing
                TFT_Set_Pen(CL_RED, 5);
                TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);
                
                // CLOSED switch
                TFT_Line(100, 155, 130, 155); 

                oldC8 = 1;
                GPIOD_ODR.B13 = 1;

               }
              else                           // detect one-to-zero transition
              {
                Delay_ms(200);
                
                // marking
                TFT_Set_Pen(CL_WHITE, 5);
                TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);
                
                // CLOSED switch
                TFT_Line(100, 155, 130, 155); 

                // showing
                TFT_Set_Pen(CL_RED, 5);
                TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);
                
                // OPEN switch
                TFT_Line(100, 155, 130, 135); 

                oldC8 = 0;
                GPIOD_ODR.B13 = 0;
              }
          }
          
          // Relay 4
          if (Button(&GPIOD_IDR, 8, 1, 1))   // detect logical one on PORTD8 pin
            {

              if (!oldD8)                    // detect zero-to-one transition
              {
                Delay_ms(200);
                
                // marking
                TFT_Set_Pen(CL_WHITE, 5);
                TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);
                
                // OPEN switch
                TFT_Line(260, 155, 290, 135); 

                // showing
                TFT_Set_Pen(CL_RED, 5);
                TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);
                
                // CLOSED switch
                TFT_Line(260, 155, 290, 155); 

                oldD8 = 1;
                GPIOA_ODR.B0 = 1;

               }
              else                           // detect one-to-zero transition
              {
                Delay_ms(200);
                
                // marking
                TFT_Set_Pen(CL_WHITE, 5);
                TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);
                
                // CLOSED switch
                TFT_Line(260, 155, 290, 155); 

                // showing
                TFT_Set_Pen(CL_RED, 5);
                TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);
                
                // OPEN switch
                TFT_Line(260, 155, 290, 135); 

                oldD8 = 0;
                GPIOA_ODR.B0 = 0;
              }
          }

      }

    }